<?php

class ServerSocket {
	var $sock;
	var $conns;
	
	function ServerSocket($ip, $port) {
		$this->sock = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		if($this->sock===FALSE) {
			throw new Exception("can't create socket");
		}
		
		$res = socket_set_option($this->sock, SOL_SOCKET, SO_REUSEADDR, 1);
		if($res==FALSE) {
			$err = socket_strerror(socket_last_error($this->sock));
			$this->close();
			throw new Exception("can't set SO_REUSEADDR: $err");
		}
		
		$res = socket_bind($this->sock, $ip, $port);
		if($res==FALSE) {
			$err = socket_strerror(socket_last_error($this->sock));
			$this->close();
			throw new Exception("can't bind socket: $err");
		}
		
		$res = socket_listen($this->sock, 20);
		if($res==FALSE) {
			$err = socket_strerror(socket_last_error($this->sock));
			$this->close();
			throw new Exception("can't listen socket: $err");
		}
		
		$res = socket_set_nonblock($this->sock);
		if($res==FALSE) {
			$err = socket_strerror(socket_last_error($this->sock));
			$this->close();
			throw new Exception("can't set non-blocking: $err");
		}
		
		$this->conns = array();
	}
	
	function created() {
		if($this->sock) return true;
		else return false;
	}
	
	function close() {
		if($this->sock) {
			socket_close($this->sock);
		}
		$this->sock = null;
	}
	
	function accept($s) {
		echo "new connection\n";
		$res = socket_set_nonblock($s);
		if($res==FALSE) {
			echo "can't set non-blocking: ".socket_strerror(socket_last_error($s))."\n";
			socket_close($s);
			return;
		}
		
		$r = array('sock'=>$s);
		$this->conns[] = $r;
		$k = array_keys($this->conns, $r);
		$k = $k[0];
		
		$this->on_accept($this->conns[$k]);
	}
	
	function loop() {
		$s = @socket_accept($this->sock);
		if($s) $this->accept($s);
		
		foreach($this->conns as $id=>$conn) {
			$s = $conn['sock'];
			$len = @socket_recv($s, $buf, 1024, MSG_DONTWAIT);
			if($len===0) {
				socket_shutdown($s);
				socket_close($s);
				unset($this->conns[$id]);
				echo "connection closed by client\n";
			}
			if($len) {
				$this->conns[$id]['buf'].=$buf;
				while(true) {
					$b1 = $this->conns[$id]['buf'];
					
					$res = $this->process($this->conns[$id]);
					if($res==false) break;
					
					$b2 = $this->conns[$id]['buf'];
					if($b1==$b2) break;
					if($b2) continue;
					break;
				}
				if($res==false) {
					socket_shutdown($s);
					socket_close($s);
					unset($this->conns[$id]);
					echo "connection closed by server\n";
				}
			}
		}
	}
	
	function on_accept(&$config) {
	}
	
	function process(&$config) {
		return false;
	}
}

?>